package ch.std.customrepo.repositories;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.data.domain.Sort;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.std.customrepo.jpa.Book;

@ExtendWith(SpringExtension.class)
@DataJpaTest
@ActiveProfiles("test") // alternativ kann mit der @TestPropertySource Annotation gearbeitet werden
public class CustomBookRepositoryJPATest {

	@Autowired
	private BookRepository bookRepository;

	private Book book;
	private List<Book> bookList;

	@BeforeEach
	public void setup() {
		book = new Book("978-1617292545", "this is the title", "that's the description", "my publisher");
		this.bookRepository.save(book);
		// insert more books
		this.bookList = new ArrayList<>();
		for (int i = 0; i < 10; i++) {
			Book localBook = null;
			if (i % 2 == 0) {
				localBook = new Book("isbn-" + i, "title-" + i, "description-" + i, "publisher-" + i);
			} else {
				localBook = new Book("isbn-".toUpperCase() + i, "title-".toUpperCase() + i,
						"description-".toUpperCase() + i, "publisher-".toUpperCase() + i);
			}
			this.bookRepository.save(localBook);
			this.bookList.add(localBook);
		}
	}

	@AfterEach
	public void tearDown() {
		this.bookRepository.deleteAllInBatch();
	}

	@Test
	public void testCustomFindByTitleContainingIgnoringCase() {
		List<Book> foundBookList = this.bookRepository.findCustomByTitle("itle");
		assertNotNull(foundBookList);
		assertEquals(11, foundBookList.size());
	}


}
